@@ -1,6 +1,6 @@
-/*	$Id: lowdown.h,v 1.82 2020/06/21 10:45:09 kristaps Exp $ */
+/*	$Id: lowdown.h,v 1.105 2021/02/18 22:37:50 kristaps Exp $ */
 /*
- * Copyright (c) 2017--2020 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) 2017--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -41,6 +41,7 @@
 #endif
 
 enum	lowdown_type {
+	LOWDOWN_GEMINI,
 	LOWDOWN_HTML,
 	LOWDOWN_LATEX,
 	LOWDOWN_MAN,
@@ -98,7 +99,7 @@ enum	lowdown_rndrt {
 struct	lowdown_buf {
 	char		*data;	/* actual character data */
 	size_t		 size;	/* size of the string */
-	size_t		 asize;	/* allocated size (0 = volatile) */
+	size_t		 maxsize; /* allocated size (0 = volatile) */
 	size_t		 unit;	/* realloc unit size (0 = read-only) */
 	int 		 buffer_free; /* obj should be freed */
 };
@@ -144,6 +145,119 @@ enum	lowdown_chng {
 	LOWDOWN_CHNG_DELETE,
 };
 
+struct	rndr_meta {
+	struct lowdown_buf key;
+};
+
+struct	rndr_paragraph {
+	size_t lines; /* input lines */
+	int beoln; /* ends on blank line */
+};
+
+struct	rndr_normal_text {
+	struct lowdown_buf text; /* basic text */
+};
+
+struct	rndr_entity {
+	struct lowdown_buf text; /* entity text */
+};
+
+struct	rndr_autolink {
+	struct lowdown_buf link; /* link address */
+	struct lowdown_buf text; /* shown address */
+	enum halink_type type; /* type of link */
+};
+
+struct	rndr_raw_html {
+	struct lowdown_buf text; /* raw html buffer */
+};
+
+struct	rndr_link {
+	struct lowdown_buf link; /* link address */
+	struct lowdown_buf title; /* title of link */
+};
+
+struct	rndr_blockcode {
+	struct lowdown_buf text; /* raw code buffer */
+	struct lowdown_buf lang; /* fence language */
+};
+
+struct	rndr_definition {
+	enum hlist_fl flags;
+};
+
+struct	rndr_codespan {
+	struct lowdown_buf text; /* raw code buffer */
+};
+
+struct	rndr_table{
+	size_t columns;
+};
+
+struct	rndr_table_header {
+	enum htbl_flags *flags;
+	size_t columns;
+};
+
+struct	rndr_table_cell {
+	enum htbl_flags flags;
+	size_t col;
+	size_t columns;
+};
+
+struct	rndr_footnote_def {
+	size_t num;
+	struct lowdown_buf key;
+};
+
+struct	rndr_footnote_ref {
+	size_t num;
+	struct lowdown_buf def;
+	struct lowdown_buf key;
+};
+
+struct	rndr_blockhtml {
+	struct lowdown_buf text;
+};
+
+struct	rndr_list {
+	/*
+	 * This should only be checked for bit-wise
+	 * HLIST_FL_ORDERED OR HLIST_FL_BLOCK.
+	 * There may be other private bits set.
+	 */
+	enum hlist_fl flags;
+	/*
+	 * This is string of size >0 iff
+	 * HLIST_FL_ORDERED and we're parsing
+	 * CommonMark, else it's an empty string.
+	 */
+	char start[10];
+};
+
+struct	rndr_listitem {
+	enum hlist_fl flags; /* all possible flags */
+	size_t num; /* index in ordered */
+};
+
+struct	rndr_header{
+	size_t level; /* hN level */
+};
+
+struct	rndr_image {
+	struct lowdown_buf link; /* image address */
+	struct lowdown_buf title; /* title of image */
+	struct lowdown_buf dims; /* NNxNN dimensions */
+	struct lowdown_buf alt; /* alt-text of image */
+	struct lowdown_buf attr_width; /* ext: width */
+	struct lowdown_buf attr_height; /* ext: height */
+};
+
+struct rndr_math {
+	struct lowdown_buf text; /* equation (opaque) */
+	int blockmode;
+};
+
 /*
  * Node parsed from input document.
  * Each node is part of the parse tree.
@@ -153,93 +267,27 @@ struct	lowdown_node {
 	enum lowdown_chng	 chng; /* change type */
 	size_t			 id; /* unique identifier */
 	union {
-		struct rndr_meta {
-			struct lowdown_buf key;
-		} rndr_meta;
-		struct rndr_list {
-			/*
-			 * This should only be checked for bit-wise
-			 * HLIST_FL_ORDERED OR HLIST_FL_BLOCK.
-			 * There may be other private bits set.
-			 */
-			enum hlist_fl flags;
-			/*
-			 * This is string of size >0 iff
-			 * HLIST_FL_ORDERED and we're parsing
-			 * CommonMark, else it's an empty string.
-			 */
-			char start[10];
-		} rndr_list; 
-		struct rndr_paragraph {
-			size_t lines; /* input lines */
-			int beoln; /* ends on blank line */
-		} rndr_paragraph;
-		struct rndr_listitem {
-			enum hlist_fl flags; /* all possible flags */
-			size_t num; /* index in ordered */
-		} rndr_listitem; 
-		struct rndr_header {
-			size_t level; /* hN level */
-		} rndr_header; 
-		struct rndr_normal_text {
-			struct lowdown_buf text; /* basic text */
-			size_t offs; /* in-render offset */
-		} rndr_normal_text; 
-		struct rndr_entity {
-			struct lowdown_buf text; /* entity text */
-		} rndr_entity; 
-		struct rndr_autolink {
-			struct lowdown_buf link; /* link address */
-			struct lowdown_buf text; /* shown address */
-			enum halink_type type; /* type of link */
-		} rndr_autolink; 
-		struct rndr_raw_html {
-			struct lowdown_buf text; /* raw html buffer */
-		} rndr_raw_html; 
-		struct rndr_link {
-			struct lowdown_buf link; /* link address */
-			struct lowdown_buf title; /* title of link */
-		} rndr_link; 
-		struct rndr_blockcode {
-			struct lowdown_buf text; /* raw code buffer */
-			struct lowdown_buf lang; /* fence language */
-		} rndr_blockcode; 
-		struct rndr_definition {
-			enum hlist_fl flags;
-		} rndr_definition; 
-		struct rndr_codespan {
-			struct lowdown_buf text; /* raw code buffer */
-		} rndr_codespan; 
-		struct rndr_table_header {
-			enum htbl_flags *flags; /* per-column flags */
-			size_t columns; /* number of columns */
-		} rndr_table_header; 
-		struct rndr_table_cell {
-			enum htbl_flags flags; /* flags for cell */
-			size_t col; /* column number */
-			size_t columns; /* number of columns */
-		} rndr_table_cell; 
-		struct rndr_footnote_def {
-			size_t num; /* footnote number */
-		} rndr_footnote_def;
-		struct rndr_footnote_ref {
-			size_t num; /* footnote number */
-		} rndr_footnote_ref;
-		struct rndr_image {
-			struct lowdown_buf link; /* image address */
-			struct lowdown_buf title; /* title of image */
-			struct lowdown_buf dims; /* NNxNN dimensions */
-			struct lowdown_buf alt; /* alt-text of image */
-			struct lowdown_buf attr_width; /* ext: width */
-			struct lowdown_buf attr_height; /* ext: height */
-		} rndr_image;
-		struct rndr_math {
-			struct lowdown_buf text; /* equation (opaque) */
-			int blockmode;
-		} rndr_math;
-		struct rndr_blockhtml {
-			struct lowdown_buf text;
-		} rndr_blockhtml;
+		struct rndr_meta rndr_meta;
+		struct rndr_list rndr_list; 
+		struct rndr_paragraph rndr_paragraph;
+		struct rndr_listitem rndr_listitem; 
+		struct rndr_header rndr_header; 
+		struct rndr_normal_text rndr_normal_text; 
+		struct rndr_entity rndr_entity; 
+		struct rndr_autolink rndr_autolink; 
+		struct rndr_raw_html rndr_raw_html; 
+		struct rndr_link rndr_link; 
+		struct rndr_blockcode rndr_blockcode; 
+		struct rndr_definition rndr_definition; 
+		struct rndr_codespan rndr_codespan; 
+		struct rndr_table rndr_table; 
+		struct rndr_table_header rndr_table_header; 
+		struct rndr_table_cell rndr_table_cell; 
+		struct rndr_footnote_def rndr_footnote_def;
+		struct rndr_footnote_ref rndr_footnote_ref;
+		struct rndr_image rndr_image;
+		struct rndr_math rndr_math;
+		struct rndr_blockhtml rndr_blockhtml;
 	};
 	struct lowdown_node *parent;
 	struct lowdown_nodeq children;
@@ -250,45 +298,57 @@ struct	lowdown_node {
  * These options contain everything needed to parse and render content.
  */
 struct	lowdown_opts {
-	enum lowdown_type	 type;
-	size_t			 maxdepth; /* max parse tree depth */
-	size_t			 cols; /* -Tterm width */
-	size_t			 hmargin; /* -Tterm left margin */
-	size_t			 vmargin; /* -Tterm top/bot margin */
-	unsigned int		 feat;
-#define LOWDOWN_TABLES		 0x01
-#define LOWDOWN_FENCED		 0x02
-#define LOWDOWN_FOOTNOTES	 0x04
-#define LOWDOWN_AUTOLINK	 0x08
-#define LOWDOWN_STRIKE		 0x10
-/* Omitted 			 0x20 */
-#define LOWDOWN_HILITE		 0x40
-/* Omitted 			 0x80 */
-#define LOWDOWN_SUPER		 0x100
-#define LOWDOWN_MATH		 0x200
-#define LOWDOWN_NOINTEM		 0x400
-/* Disabled LOWDOWN_MATHEXP	 0x1000 */
-#define LOWDOWN_NOCODEIND	 0x2000
-#define	LOWDOWN_METADATA	 0x4000
-#define	LOWDOWN_COMMONMARK	 0x8000
-#define	LOWDOWN_DEFLIST		 0x10000
-#define	LOWDOWN_IMG_EXT	 	 0x20000
-	unsigned int		 oflags;
-#define LOWDOWN_HTML_SKIP_HTML	 0x01 /* skip all HTML */
-#define LOWDOWN_HTML_ESCAPE	 0x02 /* escape HTML (if not skip) */
-#define LOWDOWN_HTML_HARD_WRAP	 0x04 /* paragraph line breaks */
-#define LOWDOWN_NROFF_SKIP_HTML	 0x08 /* skip all HTML */
-#define LOWDOWN_NROFF_HARD_WRAP	 0x10 /* paragraph line breaks */
-#define LOWDOWN_NROFF_GROFF	 0x20 /* use groff extensions */
-#define	LOWDOWN_SMARTY	  	 0x40 /* smart typography */
-#define LOWDOWN_NROFF_NUMBERED	 0x80 /* numbered section headers */
-#define LOWDOWN_HTML_HEAD_IDS	 0x100 /* <hN id="the_name"> */
-#define LOWDOWN_STANDALONE	 0x200 /* emit complete document */
-#define LOWDOWN_TERM_SHORTLINK	 0x400 /* shorten URLs */
-#define	LOWDOWN_HTML_OWASP	 0x800 /* use OWASP escaping */
-#define	LOWDOWN_HTML_NUM_ENT	 0x1000 /* use &#nn; if possible */
-#define LOWDOWN_LATEX_SKIP_HTML	 0x2000 /* skip all HTML */
-#define LOWDOWN_LATEX_NUMBERED	 0x4000 /* numbered sections */
+	enum lowdown_type	  type;
+	size_t			  maxdepth; /* max parse tree depth */
+	size_t			  cols; /* -Tterm width */
+	size_t			  hmargin; /* -Tterm left margin */
+	size_t			  vmargin; /* -Tterm top/bot margin */
+	unsigned int		  feat;
+#define LOWDOWN_TABLES		  0x01
+#define LOWDOWN_FENCED		  0x02
+#define LOWDOWN_FOOTNOTES	  0x04
+#define LOWDOWN_AUTOLINK	  0x08
+#define LOWDOWN_STRIKE		  0x10
+/* Omitted 			  0x20 */
+#define LOWDOWN_HILITE		  0x40
+/* Omitted 			  0x80 */
+#define LOWDOWN_SUPER		  0x100
+#define LOWDOWN_MATH		  0x200
+#define LOWDOWN_NOINTEM		  0x400
+/* Disabled LOWDOWN_MATHEXP	  0x1000 */
+#define LOWDOWN_NOCODEIND	  0x2000
+#define	LOWDOWN_METADATA	  0x4000
+#define	LOWDOWN_COMMONMARK	  0x8000
+#define	LOWDOWN_DEFLIST		  0x10000
+#define	LOWDOWN_IMG_EXT	 	  0x20000
+	unsigned int		  oflags;
+#define LOWDOWN_HTML_SKIP_HTML	  0x01 /* skip all HTML */
+#define LOWDOWN_HTML_ESCAPE	  0x02 /* escape HTML (if not skip) */
+#define LOWDOWN_HTML_HARD_WRAP	  0x04 /* paragraph line breaks */
+#define LOWDOWN_NROFF_SKIP_HTML	  0x08 /* skip all HTML */
+/* Disable LOWDOWN_NROFF_HARD_WRAP 0x10 */
+#define LOWDOWN_NROFF_GROFF	  0x20 /* use groff extensions */
+#define LOWDOWN_NROFF_SHORTLINK	  0x40000 /* shorten URLs */
+#define LOWDOWN_NROFF_NOLINK	  0x80000 /* don't show URLs */
+#define	LOWDOWN_SMARTY	  	  0x40 /* smart typography */
+#define LOWDOWN_NROFF_NUMBERED	  0x80 /* numbered section headers */
+#define LOWDOWN_HTML_HEAD_IDS	  0x100 /* <hN id="the_name"> */
+#define LOWDOWN_STANDALONE	  0x200 /* emit complete document */
+#define LOWDOWN_TERM_SHORTLINK	  0x400 /* shorten URLs */
+#define LOWDOWN_TERM_NOLINK	  0x20000 /* don't show URLs */
+#define	LOWDOWN_HTML_OWASP	  0x800 /* use OWASP escaping */
+#define	LOWDOWN_HTML_NUM_ENT	  0x1000 /* use &#nn; if possible */
+#define LOWDOWN_LATEX_SKIP_HTML	  0x2000 /* skip all HTML */
+#define LOWDOWN_LATEX_NUMBERED	  0x4000 /* numbered sections */
+#define	LOWDOWN_GEMINI_LINK_END	  0x8000 /* links at end */
+#define	LOWDOWN_GEMINI_LINK_IN	  0x10000 /* links inline */
+#define LOWDOWN_GEMINI_METADATA	  0x100000 /* show metadata */
+#define	LOWDOWN_GEMINI_LINK_NOREF 0x200000 /* for !inline, no names */
+#define	LOWDOWN_GEMINI_LINK_ROMAN 0x400000 /* roman link names */
+	char			**meta;
+	size_t			  metasz;
+	char			**metaovr;
+	size_t			  metaovrsz;
 };
 
 struct lowdown_doc;
@@ -300,10 +360,10 @@ __BEGIN_DECLS
  * These use the "lowdown_opts" to determine how to parse and render
  * content, and extract that content from a buffer, file, or descriptor.
  */
-void	 lowdown_buf(const struct lowdown_opts *, 
+int	 lowdown_buf(const struct lowdown_opts *, 
 		const char *, size_t,
 		char **, size_t *, struct lowdown_metaq *);
-void	 lowdown_buf_diff(const struct lowdown_opts *, 
+int	 lowdown_buf_diff(const struct lowdown_opts *, 
 		const char *, size_t, const char *, size_t,
 		char **, size_t *, struct lowdown_metaq *);
 int	 lowdown_file(const struct lowdown_opts *, 
@@ -336,31 +396,34 @@ void 	 lowdown_node_free(struct lowdown_node *);
 
 void	 lowdown_html_free(void *);
 void	*lowdown_html_new(const struct lowdown_opts *);
-void 	 lowdown_html_rndr(struct lowdown_buf *,
+int 	 lowdown_html_rndr(struct lowdown_buf *,
+		struct lowdown_metaq *, void *, 
+		const struct lowdown_node *);
+
+void	 lowdown_gemini_free(void *);
+void	*lowdown_gemini_new(const struct lowdown_opts *);
+int 	 lowdown_gemini_rndr(struct lowdown_buf *,
 		struct lowdown_metaq *, void *, 
 		const struct lowdown_node *);
 
 void	 lowdown_term_free(void *);
 void	*lowdown_term_new(const struct lowdown_opts *);
-void 	 lowdown_term_rndr(struct lowdown_buf *,
+int 	 lowdown_term_rndr(struct lowdown_buf *,
 		struct lowdown_metaq *, void *, 
 		const struct lowdown_node *);
 
 void	 lowdown_nroff_free(void *);
 void	*lowdown_nroff_new(const struct lowdown_opts *);
-void 	 lowdown_nroff_rndr(struct lowdown_buf *, 
+int 	 lowdown_nroff_rndr(struct lowdown_buf *, 
 		struct lowdown_metaq *, void *, 
-		struct lowdown_node *);
+		const struct lowdown_node *);
 
-void	 lowdown_tree_free(void *);
-void	*lowdown_tree_new(void);
-void 	 lowdown_tree_rndr(struct lowdown_buf *, 
-		struct lowdown_metaq *, void *, 
+int 	 lowdown_tree_rndr(struct lowdown_buf *, 
 		const struct lowdown_node *);
 
 void	 lowdown_latex_free(void *);
 void	*lowdown_latex_new(const struct lowdown_opts *);
-void 	 lowdown_latex_rndr(struct lowdown_buf *, 
+int 	 lowdown_latex_rndr(struct lowdown_buf *, 
 		struct lowdown_metaq *, void *, 
 		const struct lowdown_node *);
 
